const { Statement, StatementBody } = require('./statement');


function unpackQueryResultProtobuf(msg) {
  if (msg.error != null) {
    const errorMsg = msg.error.error || 'Unknown error';
    return new Error(errorMsg);
  }
  if (msg.value != null) {
    return unpackQueryResultValueProtobuf(msg.value);
  }
  throw new Error('Unexpected query result: ' + JSON.stringify(msg));
}

function unpackQueryResultValueProtobuf(msg) {
  if (msg.simple != null) {
    return unpackSimpleValue(msg.simple);
  }
  if (msg.compound != null) {
    return CompoundQueryResultValue.fromProtobuf(msg.compound);
  }
  throw new Error('Unexpected query result value ' + JSON.stringify(msg));
}

function unpackSimpleValue(val) {
  if (val.stringValue != null) return val.stringValue;
  if (val.intValue != null) return val.intValue;
  if (val.stmt != null) return Statement.fromProtobuf(val.stmt);
  if (val.stmtBody != null) return StatementBody.fromProtobuf(val.stmtBody);

  throw new Error('Unexpected query result value: ' + JSON.stringify(val));
}

class CompoundQueryResultValue {

  constructor(body) {
    this.body = body;
  }

  static fromProtobuf(msg) {
    return new CompoundQueryResultValue(msg.body.map(kv => ({
      key: kv.key,
      value: unpackSimpleValue(kv.value)
    })));
  }

  keys() {
    return this.body.map(kv => kv.key);
  }

  values() {
    return this.body.map(kv => kv.value);
  }

  statements() {
    return this.values().filter(v => v instanceof Statement).map(v => v); // dammit flow
  }
}

module.exports = {
  unpackQueryResultProtobuf,
  unpackQueryResultValueProtobuf,
  CompoundQueryResultValue
};